#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdarg.h>
#include <GL/glut.h>

static void Key( unsigned char key, int x, int y );
static void Display(void);
static void Reshape(int newwidth, int newheight);
void calc_colour(int x, int y, float *colour);
static void Mouse(int button, int state, int mouseX, int mouseY);
void handle_mouse(int x, int y);
void drawworm(int mode);

int Wwidth = 200, Wheight = 200;
int numpoints = 30, chunksize = 10;
float *pointx, *pointy;
int style = 0, numstyles = 2;


int main(int argc, char *argv[]){
	int returnval = 0, n;

	/***** initialize window *****/
	glutInit( &argc, argv );
	glutInitWindowPosition( 100, 100 );
	glutInitWindowSize(Wwidth, Wheight);

	glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );
	glutCreateWindow(argv[0]);

	/***** set up event trapping *****/
	glutReshapeFunc( Reshape );
	glutKeyboardFunc( Key );
	glutDisplayFunc( Display );
	glutMouseFunc(Mouse);
	glutPassiveMotionFunc(handle_mouse);

	/***** allocate memory ******/
	pointx = (float *)malloc(4 * numpoints);
	pointy = (float *)malloc(4 * numpoints);

	if(pointx && pointy/* && oldx && oldy*/){	
		/***** set the initial values of the points ****/
		for(n = 0; n < numpoints; n++) 
			pointx[n] = pointy[n] = n;
		
		glutMainLoop();
	}else
		returnval = 1;

	/***** free memory *****/
	if(pointx) free(pointx);
	if(pointy) free(pointy);

	return returnval;
}

static void Reshape(int newwidth, int newheight){
	Wwidth = newwidth;
	Wheight = newheight;
	glViewport(0, 0, Wwidth - 1, Wheight - 1);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, (GLint)Wwidth, 0, (GLint)Wheight);
}

static void Display(void){
	glColor3f(0, 0, 0);
	glBegin(GL_POLYGON);
		glVertex2f(0, 0);
		glVertex2f(Wwidth, 0);
		glVertex2f(Wwidth, Wheight);
		glVertex2f(0, Wheight);
	glEnd();
	glFlush();
	glutSwapBuffers();
}

static void Key( unsigned char key, int x, int y ){
	switch (key) {
		case 32:
			drawworm(0);
			style = (style + 1) % numstyles;
			drawworm(1);
			glutSwapBuffers();
			break;
		case 27:
			exit(0);
			break;
	}
}

static void Mouse(int button, int state, int mouseX, int mouseY){
	//exit(0);
}

void handle_mouse(int x, int y){
	int mousex = x, mousey = Wheight - y + 1, n;
	float slope, dist;
	/****** erase the old segments ******/
	drawworm(0);
	/*************** Move the worm *****************/

	/**** move the first point up to the mouse ****/
	pointx[0] = mousex;
	pointy[0] = mousey;

	/***** make the rest of the points follow *****/
	for(n = 1; n < numpoints; n++){
		dist = sqrt(fabs((pointx[n] - pointx[n - 1]) * (pointx[n] - pointx[n - 1]) + 
				(pointy[n] - pointy[n - 1]) * (pointy[n] - pointy[n - 1])
			));

		if(dist != 0){
			slope = (float)chunksize / dist;
			pointx[n] = - slope * (pointx[n - 1] - pointx[n]) + pointx[n - 1];
			pointy[n] = - slope * (pointy[n - 1] - pointy[n]) + pointy[n - 1];
		}
	}

	drawworm(1);

	/***********************************************/

	glFlush();
	glutSwapBuffers();
}

void drawworm(int mode){
	int n, px, py;
	float f;
	/****** draw the new segments ******/
	for(n = numpoints - 1; n > 0; n--){
		if(mode){
			f = (float)(numpoints - n) / (float)numpoints;
			glColor3f(f + .5, f / 2., 0.);
		}else glColor3f(0, 0, 0);

		if(style == 0){
			glBegin(GL_LINE_STRIP);			
				glVertex2f(pointx[n], pointy[n]);
				glVertex2f(pointx[n - 1], pointy[n - 1]);
			glEnd();
		}else{
			glBegin(GL_POLYGON);
				px = pointx[n] + 4. * (pointx[n] - pointx[n - 1]);
				py = pointy[n] + 4. * (pointy[n] - pointy[n - 1]);
				glVertex2f(px, py);

				px = pointx[n - 1] + (pointy[n - 1] - pointy[n]) / 3.;
				py = pointy[n - 1] - (pointx[n - 1] - pointx[n]) / 3.;
				glVertex2f(px, py);

				px = pointx[n - 1] - (pointy[n - 1] - pointy[n]) / 3.;
				py = pointy[n - 1] + (pointx[n - 1] - pointx[n]) / 3.;
				glVertex2f(px, py);
			glEnd();
		}
	}
}
